/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: $
  Language:  C++
  Date:      $Date: $
  Version:   $Revision: $
  Author:    Gavin Baker <gavinb@cs.mu.oz.au>

  Copyright 2004 (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __itkAdditiveGaussianNoiseImageFilter_h
#define __itkAdditiveGaussianNoiseImageFilter_h

#include "itkImageToImageFilter.h"
#include "itkNormalVariateGenerator.h"
#include "itkUnaryFunctorImageFilter.h"

namespace itk
{

/** \class NormalVariateMultiplierFunctor
 * \brief Pixel functor that adds Gaussian noise
 */
template < class InputPixelType >
class NormalVariateNoiseFunctor
{
  public:

    NormalVariateNoiseFunctor()
    {
        generator = itk::Statistics::NormalVariateGenerator::New();

        SetSeed( 42 );
    }

    void SetSeed( unsigned long seed )
    {
        generator->Initialize( seed );
    }

    void SetAlpha( float alpha )
    {
        this->m_Alpha = alpha;
    }

    float GetAlpha() const
    {
        return m_Alpha;
    }

    InputPixelType operator()( InputPixelType input )
    {
        const float min = itk::NumericTraits< InputPixelType >::min();
        const float max = itk::NumericTraits< InputPixelType >::max();
        const float mid = max / 2.0;

        float out = (float)input + m_Alpha * mid * generator->GetVariate();

        // Clamp the output value in valid range
        out = ( out < min ? min : out );
        out = ( out > max ? max : out );

        return static_cast< InputPixelType > ( out );
    }

private:

    itk::Statistics::NormalVariateGenerator::Pointer generator;
    float m_Alpha;
};


/** \class AdditiveGaussianNoiseImageFilter
 * \brief Adds Gaussian noise to the input image
 *
 * Adds noise to the input image, over the full dynamic range
 * of the data type, according to a Gaussian normal variate distribution.
 * The amount of noise is determined by the \alpha parameter,
 * which is a multiplier for the generator.
 *
 * \ingroup ImageToImageFilters
 */
template <class TInputImage >
class ITK_EXPORT AdditiveGaussianNoiseImageFilter :
    public ImageToImageFilter< TInputImage, TInputImage >
{
public:
  /** Standard class typedefs. */
  typedef AdditiveGaussianNoiseImageFilter Self;
  typedef ImageToImageFilter<TInputImage, TInputImage>  Superclass;
  typedef SmartPointer<Self>  Pointer;
  typedef SmartPointer<const Self>  ConstPointer;

  /** Method for creation through the object factory */
  itkNewMacro(Self);  

  /** Run-time type information (and related methods). */
  itkTypeMacro(AdditiveGaussianNoiseImageFilter, ImageToImageFilter);

  /** Superclass typedefs. */
  //typedef typename Superclass::OutputImageRegionType OutputImageRegionType;

  /** Some convenient typedefs. */
  typedef TInputImage InputImageType;
  typedef typename InputImageType::Pointer        InputImagePointer;
  typedef typename InputImageType::ConstPointer   InputImageConstPointer;
  typedef typename InputImageType::RegionType     InputImageRegionType; 
  typedef typename InputImageType::PixelType      InputImagePixelType; 
  
  /** ImageDimension constants */
  itkStaticConstMacro(InputImageDimension, unsigned int,
                      TInputImage::ImageDimension);

  void GenerateData();

  void SetAlpha( float alpha )
  {
      m_NoiseFilter->GetFunctor().SetAlpha( alpha );
  }

  float GetAlpha() const
  {
      return m_NoiseFilter->GetFunctor().GetAlpha();
  }

 protected:

  AdditiveGaussianNoiseImageFilter();
  //~AdditiveGaussianNoiseImageFilter();

  virtual void PrintSelf(std::ostream& os, Indent indent) const;

private:

  AdditiveGaussianNoiseImageFilter(const Self&);  // intentionally not implemented
  void operator=(const Self&);      // intentionally not implemented

  public:

  typedef UnaryFunctorImageFilter<
      InputImageType,
      InputImageType,
      NormalVariateNoiseFunctor< typename InputImageType::PixelType > >
          NoiseFilterType;

  private:

  typename NoiseFilterType::Pointer m_NoiseFilter;
};

} /* end namespace itk */

#ifndef ITK_MANUAL_INSTANTIATION
#include "itkAdditiveGaussianNoiseImageFilter.txx"
#endif

#endif /* __itkAdditiveGaussianNoiseImageFilter_h */
